
/*
   helevent.c
   
   Event routines and some dialog routines for helicon.
   
   Author: Mark E. Nash
   Date:   July, 1988
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<malloc.h>
#include	<string.h>
#include	<conio.h>
#include	<graph.h>
#include	<ctype.h>
#include	<dos.h>
#include	<math.h>
#include	"helimacs.h"
#include	"helimess.h"
#include	"helstruc.h"
#include	"helfdef.h"


/* externals */
extern char msg[];			/* in heliglob.h */
extern short cxpos;			/* in heliglob.h */
extern short cypos;			/* in heliglob.h */
extern struct dialog *dlogtop;		/* in heliglob.h */
extern unsigned int delcon;		/* in heliglob.h */
extern unsigned mouse;			/* in heliglob.h */
extern unsigned m1, m2, m3, m4;		/* in heliglobs.h */


int GetNextEvent(key)
	char *key;
/*
   GetNextEvent() gets the next event.
   
   Entry: A GetNextEvent has been requested.
          The global variable mouse is current.
   
   Exit:  The next event has been returned.
          key points to the key that was hit when in keyboard mode.
*/
	{
	int	event;			/* the current event */
	int	funkey;			/* for funky keys */
	
	
	event = NULLEVENT;

	if (mouse)
		{	/* mouse */
/* see if a key has been selected */
		funkey = FALSE;
		if (kbhit())
			{
			*key = (char)getch();
			if (*key == 0)
				{
				*key = (char)getch();
				funkey = TRUE;
				}
			}
/* check for cursor and select event */
		MOUSE(3, 0, 0, 0);
		event = CURUPDATE;
		cxpos = m3;
		cypos = m4;
		if (funkey)
			if (*key == ALT)
				event = ALTEVENT;/* alt overrides cursor */
		if (m2 != 0)
			event = SELECT;		/* select overrides all */
		}

	else		/* cursor keys */
		{
		funkey = FALSE;
		*key = (char)getch();
		if (*key == 0)
			{
			*key = (char)getch();
			funkey = TRUE;
			}

		switch (*key)
			{
			case UP:
			case DOWN:
			case RIGHT:
			case LEFT:
			case HOME:
				event = CURUPDATE;
				break;
			
			case ALT:
				if (funkey)
					event = ALTEVENT;
				break;
				
			case RETURN:
				event = SELECT;
				break;
			}		
		}
		
	return(event);
	}
	
	
	
int setcpos(mt, key)
	struct menutop *mt;
	char key;
/*
   setcpos() sets the cursor position if in keyboard mode.
   
   Entry: A CURUPDATE event in the keyboard mode was received.
          mt points to the menu structure.
          key is the update key direction.
	  (cxpos,cypos) is the current position of the cursor.
   
   Exit:  (cxpos,cypos) are updated.
   
   Note:  This routine is needed so that a mouse and keyboard can be used.  The
          keyboard cursor is different because it must be restricted to the
	  menu area, while the mouse cursor can roam freely.
*/
	{
	static struct menu *mp = NULL;		/* point to current menu */
	static struct menuitem *ip = NULL;	/* point to cur. menu item */
	struct menu *mr;			/* run through the menu */
	struct menuitem *ir;			/* run through the items */
	struct menu *lastm;			/* previous menu pointed to */
	struct menuitem *lasti;			/* previous item pointed to */
	

	if (mt == NULL)
		return(ERR);
		
/* check for the outmenu key */
	if (key == OUTMENU)
		{
		cxpos = 0;
		cypos = 0;
		mp = NULL;
		ip = NULL;
		return(NOERR);
		}

/* check for home key */
	if (key == HOME)
		{
		mp = mt->start;
		ip = NULL;
		cxpos = mt->start->box.x1 + 1;
		cypos = mt->start->box.y1 + 1;
		mp = mt->start;
		ip = NULL;
		return(NOERR);
		}
		
/* now find out where the cursor is */
/* menu, if any */
	mr = mt->start;
	lastm = NULL;
	while (mr != NULL && mr != mp)
		{
		lastm = mr;
		mr = mr->next;
		}
		
/* item, if any */
	if (mr != NULL)
		{
		ir = mr->top;
		lasti = NULL;
		while (ir != NULL && ir != ip)
			{
			lasti = ir;
			ir = ir->next;
			}
		}
			
/* move it to where it wants to be */
/* nowhere */
	if (mr == NULL)
		{
		cxpos = 0;
		cypos = 0;
		mp = NULL;
		ip = NULL;
		}

/* somewhere, but wants to move */
	else
		switch (key)
			{
			case UP:       /* once in menu, can't go to menu bar */
				if (ir != NULL)
					if (lasti != NULL)
						{
						cypos = lasti->box.y1 + 1;
						ip = lasti;
						}
				break;

			case DOWN:
				if (ir == NULL)		/* in menu bar */
					{
					ip = mr->top;
					if (ip != NULL)
						cypos = ip->box.y1 + 1;
					}

				else		/* bottom or no items */
					if (ip != NULL)
						if (ip->next != NULL)
							{
							ip = ip->next;
							cypos = ip->box.y1 + 1;
							}
				break;

			case RIGHT:
				if (mr != NULL)
					if (mr->next != NULL)
						{
						mr = mr->next;
						cxpos = mr->box.x1 + 1;
						cypos = mr->box.y1 + 1;
						mp = mr;
						ip = NULL;
						}
				break;

			case LEFT:
				if (lastm != NULL)
					{
					cxpos = lastm->box.x1 + 1;
					cypos = lastm->box.y1 + 1;
					mp = lastm;
					ip = NULL;
					}
				break;

			default:
				return(ERR);
			}

	return(NOERR);
	}
	
	
	
int chkpop(mt)
	struct menutop *mt;
/*
   chkpop() returns the id of any menu that has been pulled down.
   
   Entry: A request for pulled down menus has been made.
   
   Exit:  Returns the id of any pulled down menu, else NULLID.
*/
	{
	struct menu *mp;
	
	
	if (mt == NULL)
		return(NULLID);
		
	mp = mt->start;
	while (mp != NULL && mp->pd == FALSE)
		mp = mp->next;
		
	if (mp != NULL)
		return(mp->menuid);
	else
		return(NULLID);
	}
	
	
	
int chkbar(mt)
	struct menutop *mt;
/*
   chkbar() checks to see if the cursor is in the menu bar.
   
   Entry: mt points to the menu top.
          Need to know if the cursor is in a menu region in the menu bar.
   
   Exit:  Returns menu id if in region, else NULLID.
*/
	{
	struct menu *mp;		/* point to a menu */
	unsigned id;			/* menu id */
	static int mhid = NULLID;	/* menu now hilighted */
	
	
	id = NULLID;
	if (mt == NULL)
		return(id);
		
	mp = mt->start;
	while (mp != NULL && id == NULLID)
		{
		if (curnrgn(&(mp->box)))
			id = mp->menuid;
		mp = mp->next;
		}
			
/* do hilighting here */
	if (id != mhid)
		{
/* unhilight old */
		if (mhid != NULLID)
			hilsel(mt, mhid, NULLID, FALSE);
/* hilight new */
		mhid = id;
		if (mhid != NULLID)
			hilsel(mt, mhid, NULLID, FALSE);
		}

	return(id);
	}
	
	
	
int chkmenu(mt, mid)
	struct menutop *mt;
	int mid;
/*
   chkmenu() checks to see if the cursor is in a pulled down menu.

   Entry: mt is the top of the menu structure.
          mid is the id of the current menu.
          Need to know if the cursor is in a menu region.
   
   Exit:  Returns menu id if in an item region, else NULLID.
*/
	{
	struct menu *mp;		/* point to a menu */
	struct menuitem *ip;		/* point to an item */
	unsigned id;			/* menu item id */
	static int omenu = NULLID;	/* menu now hilighted */
	static int ihid = NULLID;	/* item now hilighted */
	
	
	if (mt == NULL)
		return(NULLID);
		
/* point to the current menu */
	mp = mt->start;
	while (mp != NULL && mp->menuid != mid)
		mp = mp->next;
		
	if (mp == NULL)
		return(NULLID);

/* check to see if cursor is in an item region */
	id = NULLID;
		
	ip = mp->top;
	while (ip != NULL && id == NULLID)
		{
		if (curnrgn(&(ip->box)))
			id = ip->itemid;
		ip = ip->next;
		}

/* do hilighting here */
	if (id != ihid && mid == omenu)
		{
/* unhilight old */
		if (ihid != NULLID)
			hilsel(mt, mid, ihid, FALSE);
/* hilight new */
		ihid = id;
		if (ihid != NULLID)
			hilsel(mt, mid, ihid, FALSE);
		}

/* new menu, therefore it must be a new item, although may have same id */
	if (mid != omenu)
		{
		omenu = mid;
/* hilight new */
		ihid = id;
		if (ihid != NULLID)
			hilsel(mt, mid, ihid, FALSE);
		}
		
	return(id);
	}
	
	
	
int curnrgn(region)
	struct hibox *region;
/*
   curnrgn() tells whether the cursor is in a given region.
   
   Entry: region points to the region to check.
          the cursor is at (cxpos,cypos).
	  
   Exit:  Returns TRUE if cursor is in region, else FALSE.
*/
	{
	if (region != NULL)
		if (cxpos >= region->x1)
			if (cxpos <= region->x2)
				if (cypos >= region->y1)
					if (cypos <= region->y2)
						return(TRUE);
	return(FALSE);
	}
	
	
	
struct dialog *do_dialog(id)
	int id;
/*
   do_dialog() initiates a dialog with the user.
   
   Entry: A dialog needs to be initiated.
          id is the dialog to initiate.
	  
   Exit:  The user has ended the dialog.
          Dialog information has been returned in the eboxes of the dialog.
	  Returns a pointer to the dialog box.
*/
	{
	struct dialog *curlog;			/* point to a dialog */
	struct diatext *txt;			/* point to dialog text */
	struct editbox *ebx;			/* point to edit box */
	struct grafwindow *dw;			/* dialog window */
	
	
	if (id == NULLID)
		{
		sprintf(msg, L_NULLID, "do_dialog()");
		err_window(msg);
		return(NULL);
		}
		
/* find id */
	curlog = dlogtop;
	while (curlog != NULL && curlog->id != id)
		curlog = curlog->next;
		
	if (curlog == NULL)
		{
		sprintf(msg, L_NOID, id, "do_dialog()");
		err_window(msg);
		return(NULL);
		}
		
/* open the dialog window */
	dw = open_grafwin(TEXTWIN, curlog->dlx1, curlog->dly1, 0, 0, DLBRCOLOR,
				DLBKCOLOR, curlog->dcols, curlog->drows, TRUE);
	
/* print the dialog text */
	txt = curlog->dtext;
	while (txt != NULL)
		{
		gtwcurpos(dw, txt->col, txt->row);
		eputs_window(dw, txt->text, DLTXTCOLOR, FALSE);
		txt = txt->next;
		}
		
/* display the dialog eboxes */
	ebx = curlog->ebox;
	while (ebx != NULL)
		{
		ebx->ebw = open_grafwin(TEXTWIN, ebx->lx1, ebx->ly1, 0,
				0, EBBRCOLOR, EBBKCOLOR, ebx->size, 1, FALSE);
		eputs_window(ebx->ebw, ebx->tbox, EBTXCOLOR, FALSE);
		gtwcurpos(ebx->ebw, 0, 0);
		ebx = ebx->next;
		}
	
/* do a dialog edit */
	if (dedit(curlog->ebox, dw) == ERR)
		err_window(L_BADED);
	
/* close all of the windows */
	ebx = curlog->ebox;
	while (ebx != NULL)
		{
		close_grafwin(&(ebx->ebw));
		ebx = ebx->next;
		}
		
	close_grafwin(&dw);
	
	return(curlog);
	}
	
	
	
int dedit(topebx, dlw)
	struct editbox *topebx;
	struct grafwindow *dlw;
/*
   dedit() allows editing in a dialog.
   
   Entry: topebx points to the first dialog ebox in the list.
          dlw points to the dialog window.
   
   Exit:  The editing session is finished.
          The information for each ebox is in it's ebox field.
*/
	{
	struct editbox *cb, *teb, *leb; /* current, temp, and last eboxes */
	int	done;			/* finished flag */
	int	ch;			/* last character read */
	int	ech;			/* for funny keys */
	unsigned short ccol, crow;	/* display cursor position */
	char	tstr[TBOXSIZE];		/* temporary string for editing */
	unsigned delctr1, delctr2;	/* blink delay counter */
	struct xycoord xy;		/* for coord conv. */


/* this is basically an event driver */
	if (kbhit())
		getch();	/* clear out keyboard buffer */
	cb = topebx;
	win_cur(cb->ebw);
	done = FALSE;
	while (!done)
		{
/* check the mouse status */
		MOUSE(3, 0, 0, 0);
		xy = _getlogcoord(m3, m4);
		cxpos = xy.xcoord;
		cypos = xy.ycoord;
		if (m2 != 0)
			if (curnrgn(&(dlw->wbox)))
				done = TRUE;

		ch = NULLKEY;
		ech == FALSE;
		if (kbhit() && !done)
			{
			ch = getch();
			if (ch == 0)
				{
				ch = getch();
				ech = TRUE;
		/* remap funkey keys */
				switch (ch)
					{
					case UP:
						ch = UPKEY;
						break;
					case DOWN:
						ch = DOWNKEY;
						break;
					case LEFT:
						ch = LEFTKEY;
						break;
					case RIGHT:
						ch = RIGHTKEY;
						break;
					case DEL:
						ch = DELKEY;
						break;
					default:
						break;
					}
				}
			}
		tstr[0] = '\0';		/* init edit string */
		switch (ch)
			{
			case UPKEY:
		/* erase old cursor */
				win_cur(cb->ebw);
		/* change the ebox pointer */
				if (cb != topebx)
					{
					leb = topebx;
					teb = topebx->next;
					while (teb != cb && teb != NULL)
						{
						leb = teb;
						teb = teb->next;
						}
					cb = leb;
					}
		/* show the cursor */
				win_cur(cb->ebw);
				break;
			
			case DOWNKEY:
			case RETURN:
		/* erase old cursor */
				win_cur(cb->ebw);
		/* change the ebox pointer */
				if (cb->next != NULL)
					cb = cb->next;
				else
					if (ch == RETURN)
						cb = topebx;
		/* show the cursor */
				win_cur(cb->ebw);
				break;
			
			case LEFTKEY:
		/* erase old cursor */
				win_cur(cb->ebw);
		/* get cursor pos */
				gtwgetpos(cb->ebw, &ccol, &crow);
				if (ccol > 0)
					{
					ccol--;
					gtwcurpos(cb->ebw, ccol, crow);
					}
				else
					printf("\7");
		/* display new cursor */
				win_cur(cb->ebw);
				break;
			
			case RIGHTKEY:
		/* erase old cursor */
				win_cur(cb->ebw);
		/* get cursor pos */
				gtwgetpos(cb->ebw, &ccol, &crow);
				if (ccol < strlen(cb->tbox))
					{
					ccol++;
					gtwcurpos(cb->ebw, ccol, crow);
					}
				else
					printf("\7");
		/* display new cursor */
				win_cur(cb->ebw);
				break;
			
			case DELKEY:
				win_cur(cb->ebw);
		/* get the cursor pos */
				gtwgetpos(cb->ebw, &ccol, &crow);
				if (ccol <= strlen(cb->tbox))
					{
		/* erase old display */
					eputs_window(cb->ebw,
							&(cb->tbox[ccol]),
							EBBKCOLOR, FALSE);
					gtwcurpos(cb->ebw, ccol, crow);
		/* set the info string */
					strcat(tstr, &(cb->tbox[ccol+1]));
					cb->tbox[ccol] = '\0';
					strcat(cb->tbox, tstr);
		/* re-display string */
					eputs_window(cb->ebw,
							&(cb->tbox[ccol]),
							EBTXCOLOR, FALSE);
					gtwcurpos(cb->ebw, ccol, crow);
					}
				else
					printf("\7");

				win_cur(cb->ebw);
				break;
			
			case BACKSPACE:
				win_cur(cb->ebw);
		/* get the cursor pos */
				gtwgetpos(cb->ebw, &ccol, &crow);
				if (ccol > 0)
					{
		/* set the new cursor pos */
					ccol--;
					gtwcurpos(cb->ebw, ccol, crow);
		/* erase old display */
					eputs_window(cb->ebw,
						      &(cb->tbox[ccol]),
							EBBKCOLOR, FALSE);
					gtwcurpos(cb->ebw, ccol, crow);
		/* set the info string */
					strcat(tstr, &(cb->tbox[ccol+1]));
					cb->tbox[ccol] = '\0';
					strcat(cb->tbox, tstr);
		/* re-display string */
					eputs_window(cb->ebw,
							&(cb->tbox[ccol]),
							EBTXCOLOR, FALSE);
					gtwcurpos(cb->ebw, ccol, crow);
					}
				else
					printf("\7");
					
				win_cur(cb->ebw);
				break;
			
			case ESC:		/* get out of dialog */
				done = TRUE;
				break;
			
			default:		/* regular keystroke */
/* check the mouse status */
				MOUSE(3, 0, 0, 0);
				cxpos = m3;
				cypos = m4;
				if (m2 != 0)
					if (curnrgn(&(dlw->wbox)))
						{
						done = TRUE;
						break;
						}
				win_cur(cb->ebw);
				if (isprint(ch))
					{
					if (strlen(cb->tbox) + 1 >=
								cb->size)
						printf("\7");
					else
						{
						gtwgetpos(cb->ebw, &ccol,
									&crow);
				/* erase old display */
						eputs_window(cb->ebw,
							&(cb->tbox[ccol]),
							EBBKCOLOR, FALSE);
						gtwcurpos(cb->ebw, ccol, crow);
				/* set the new string */
						if (cb->tbox[ccol] == '\0')
						   {
						   cb->tbox[ccol] = (char)ch;
						   cb->tbox[ccol+1] = '\0';
						   }
						else
						   {
						   strcat(tstr,
							&(cb->tbox[ccol]));
						   cb->tbox[ccol] = (char)ch;
						   cb->tbox[ccol+1] = '\0';
						   strcat(cb->tbox, tstr);
						   }
						gtwcurpos(cb->ebw, ccol, crow);
				/* update the display */
						eputs_window(cb->ebw,
							&(cb->tbox[ccol]),
							EBTXCOLOR, FALSE);
				/* inc the column */
						ccol++;
						gtwcurpos(cb->ebw, ccol, crow);
						}
					}
			/* delay loop */
				if (ch == NULLKEY)
#pragma loop_opt(off)
					for (delctr2=0; delctr2<CBLMAX; 
								delctr2++)
						{
				/* check the mouse status */
						MOUSE(3, 0, 0, 0);
						xy = _getlogcoord(m3, m4);
						cxpos = xy.xcoord;
						cypos = xy.ycoord;
						if (m2 != 0)
							if (curnrgn(
								&(dlw->wbox)))
								{
								done = TRUE;
								break;
								}
						if (kbhit())
							break;
						for (delctr1=0; 
						    delctr1<delcon; delctr1++);
						}
				win_cur(cb->ebw);
			/* delay loop */
				if (ch == NULLKEY)
					for (delctr2=0; delctr2<CBLMAX; 
								delctr2++)
						{
				/* check the mouse status */
						MOUSE(3, 0, 0, 0);
						xy = _getlogcoord(m3, m4);
						cxpos = xy.xcoord;
						cypos = xy.ycoord;
						if (m2 != 0)
							if (curnrgn(
								&(dlw->wbox)))
								{
								done = TRUE;
								break;
								}
						if (kbhit())
							break;
						for (delctr1=0; 
						    delctr1<delcon; delctr1++);
						}
#pragma loop_opt()
				break;
			}
		}

	return(NOERR);
	}
	
	
	
int fmouse()
/*
   fmouse() calls the software interrupt to do a mouse function.
   
   Entry: p1-p4 point to the mouse parameters to use.
   
   Exit:  p1-p4 have been set according to the function.
*/
	{
	union REGS regs;		/* in dos.h */
	

	if (mouse)
		{
		regs.x.ax = m1;
		regs.x.bx = m2;
		regs.x.cx = m3;
		regs.x.dx = m4;
		int86(MOUSEINT, &regs, &regs);
		m1 = regs.x.ax;
		m2 = regs.x.bx;
		m3 = regs.x.cx;
		m4 = regs.x.dx;
		}
		
	return(NOERR);
	}
	
	
	
int bdist()
/*
   bdist() initiates the building of a custom distribution.
   
   Entry: The estimate command has been selected.
          The global dw points to the data window.
   
   Exit:  A custom distribution has been built.
*/
	{
	double	xmin, xmax, xmed, xlow, xhi;	/* data window dist. calc. */
	double	sxmin, symin, sxmax, symax;	/* data window scale */
	static double sect[CSECT*CPTS];		/* four dist. sections */
	double	slimit[CSECT+1];		/* bin limits for c. dist. */
	double	pt;				/* randomly generated point */
	int	ctr, ctr1, ctr2, ctr3;		/* gp counters */
	struct rawcell *rd, *prev;	/* sorting pointers */
	struct rawcell *rdtop;		/* top raw data in the list */
	struct rawcell *data;		/* data that has been read */
	struct grafwindow *ms;		/* message window */
	extern struct grafwindow *dw;		/* heliglob.h */
	extern struct grafwindow *pw;		/* heliglob.h */
	extern struct grafwindow *iw;		/* heliglob.h */
	extern char tboxes[NUMTBXS][TBOXSIZE+1];/* heliglob.h */
	extern struct rawcell *rd_top;		/* heliglob.h */
	struct menu *mp;		/* to stroll through the menu list */
	struct menuitem *ip;		/* to stroll through the item list */
	extern struct menutop *menus;			/* in heliglob.h */
	extern double params[MAXPARS+1][NUMDIST+1];	/* in heliglob.h */
	extern char clrstr[];				/* in heliglob.h */

	
/* erase screen */
	win_clear(pw, IBKCOLOR);
	win_clear(dw, IBKCOLOR);

/* clear file name */
	gtwcurpos(iw, IWFNCOL, 1);
	clrstr[20] = '\0';
	eputs_window(iw, clrstr, IBKCOLOR, FALSE);
/* clear current fit name */
	gtwcurpos(iw, IWFNCOL, 3);
	eputs_window(iw, clrstr, IBKCOLOR, FALSE);
	clrstr[20] = '\200';

/* reset last analysis */
/* reset FITQ flags and error values when bins are re-made */
	for (ctr=NULLDIST+1; ctr<=NUMDIST; ctr++)
		{
		params[FITQ][ctr] = DOFIT;
		params[MSE][ctr] = -1.0;
		}

/* reset fit menu hilights */
	mp = menus->start;
	while (mp->menuid != FITID)
		mp = mp->next;

	ip = mp->top;
	while (ip != NULL)
		{
		ip->hf = FALSE;
		ip = ip->next;
		}
	
/* get the min-max, scale the x-axis, and display */
	do
		{
		do_dialog(BMINMAX);
		xmin = atof(tboxes[BMIN]);
		xmax = atof(tboxes[BMAX]);
		if (xmin >= xmax)
			{
			sprintf(msg, D_BMINMAX, xmin, xmax);
			err_window(msg);
			}
		} while (xmin >= xmax);
		
	sxmin = xmin - BSCONST * (xmax - xmin);
	symin = 0.0;
	sxmax = xmax + SCONST * (xmax - xmin);
	symax = 10.0;

	if (scale_grafwin(dw, sxmin, symin, sxmax, symax) == ERR)
		{
		err_window(W_BADSCALE);
		return(ERR);
		}
		
/* draw scale */
	win_dscale(dw, SCALECOLOR, XAXIS);
	
/* show the min, max */
	win_line(dw, xmin, dw->ysmin, xmin, dw->ysmin + 
			(dw->ysmax - dw->ysmin) * (MAJTIC+0.01), MEANCOLOR);

	win_line(dw, xmax, dw->ysmin, xmax, dw->ysmin + 
			(dw->ysmax - dw->ysmin) * (MAJTIC+0.01), MEANCOLOR);
	
/* get the median and display */
	do
		{
		do_dialog(BMEDD);
		xmed = atof(tboxes[BMED]);
		if (xmed <= xmin || xmed >= xmax)
			{
			sprintf(msg, D_BMED, xmed, xmin, xmax);
			err_window(msg);
			}
		} while (xmed <= xmin || xmed >= xmax);
		
/* show the median */
	win_line(dw, xmed, dw->ysmin, xmed, dw->ysmin + 
			(dw->ysmax - dw->ysmin) * (MAJTIC+0.01), MEANCOLOR);

/* get the sub-medians and display */
	do
		{
		do_dialog(BSUBMED);
		xlow = atof(tboxes[BLOW]);
		xhi = atof(tboxes[BHI]);
		if (xlow <= xmin || xlow >= xmed)
			{
			sprintf(msg, D_BSMED, xlow, xmin, xmed);
			err_window(msg);
			}
		if (xhi <= xmed || xhi >= xmax)
			{
			sprintf(msg, D_BSMED, xhi, xmed, xmax);
			err_window(msg);
			}
		} while (xlow <= xmin || xlow >= xmed || xhi <= xmed || 
								xhi >= xmax);
		
/* show the sub-medians */
	win_line(dw, xlow, dw->ysmin, xlow, dw->ysmin + 
			(dw->ysmax - dw->ysmin) * (MAJTIC+0.01), MEANCOLOR);

	win_line(dw, xhi, dw->ysmin, xhi, dw->ysmin + 
			(dw->ysmax - dw->ysmin) * (MAJTIC+0.01), MEANCOLOR);
			
/* set up the custom distribution limits */
	slimit[0] = xmin;
	slimit[1] = xlow;
	slimit[2] = xmed;
	slimit[3] = xhi;
	slimit[4] = xmax;
	
/* display point generation message */
	if ((ms = msg_window(I_PGEN)) == NULL)
		{
		printf("\7\7\7");
		return(ERR);
		}
/* generate points for each section of the distribution */
	ctr3 = 0;
	for (ctr1=0; ctr1<CSECT; ctr1++)
		for(ctr2=0; ctr2<CPTS; ctr2++)
			{
			pt = (slimit[ctr1+1] - slimit[ctr1]) * ((float)rand() 
						/ 32767.0) + slimit[ctr1];
			if (pt <= slimit[ctr1] || pt >= slimit[ctr1+1])
				{
				sprintf(msg, D_BDGEN, pt, slimit[ctr1], 
							slimit[ctr1+1]);
				err_window(msg);
				}
			else
				{
				sect[ctr3] = pt;
				ctr3++;
				}
			}

/* now put these points in the raw data list */
	free_rawdat(&rd_top);
	rdtop = NULL;
	ctr3 = 0;
	if ((rdtop = (struct rawcell *)malloc(sizeof(struct rawcell))) == NULL)
		{
		printf("\7");
		return(ERR); 
		}
/* update memory status */
	memupdt((unsigned long)sizeof(struct rawcell), ALLOC);

	rd = rdtop;
	rd->x = sect[ctr3];
	rd->next = NULL;

/* generate the following data and sort */
	ctr3++;
	rd = rdtop;
	prev = NULL;
	while (ctr3 < CSECT*CPTS)
		{
/* allocate storage for the data generated */
		if ((data = (struct rawcell *)malloc(sizeof(struct rawcell)))
								 == NULL)
			{
			printf("\7");
			return(ERR);
			}
/* update memory status */
		memupdt((unsigned long)sizeof(struct rawcell), ALLOC);

		data->x = sect[ctr3];
		data->next = NULL;
			
		if (sect[ctr3] < rd->x)	/* start at beginning */
			{
			rd = rdtop;
			prev = NULL;
			if (sect[ctr3] < rdtop->x)	/* new start */
				{
				data->next = rdtop;
				rdtop = data;
				}

			else			/* search for start */
				{
				while (rd->next != NULL && sect[ctr3] >= rd->x)
					{
					prev = rd;
					rd = rd->next;
					}
				data->next = rd;
				prev->next = data;
				}
			}

		else		/* insert at interior or end */
			{
	/* get to the spot we want */
			while (rd->next != NULL && sect[ctr3] >= rd->x)
				{
				prev = rd;
				rd = rd->next;
				}
				
			if (rd->x > sect[ctr3]) /* can happen at end of list */
				{
				prev->next = data;
				data->next = rd;
				prev = rd;
				}
			else
				{
				rd->next = data;
				prev = rd;
				rd = rd->next;
				}
			}
		ctr3++;
		}

/* assign the list */
	rd_top = rdtop;
	
/* close the message window now */
	close_grafwin(&ms);

/* reset the default number of bins */
	sprintf(msg, "%d", BDNUMBINS);
	strcpy(tboxes[NBINS], msg);
	win_clear(dw, IBKCOLOR);
	plotdat(CONTINUOUS);

	return(NOERR);
	}
