
/*
   heliglob.h
   
   Global variables for Helicon.
   
   Author: Mark E. Nash
   Date:   May, 1988
*/

/* graph.h must be included a priori this file */

struct videoconfig far vconfig;		/* video configuration data */

unsigned int delcon;			/* delay constant for this machine */

struct rawcell *rd_top = NULL;		/* top of raw data list */
struct topbin *bintop = NULL;		/* top of bin list */
struct grafwindow *top_window = NULL;	/* top of window list */
struct menutop *menus = NULL;		/* point to menu structure */
struct dialog *dlogtop = NULL;		/* top of dialog list */

struct mwdat mem;			/* memory monitor data */
int	memon = FALSE;			/* monitor memory or not */
unsigned long mopap = 0l,	/* size of helicon */
	      mfree = 0l,	/* free memory */
	      mall = 0l,	/* allocated memory */
	      mtot = 0l;	/* total memory kbytes */

char msg[MESSLEN + 1];			/* messages */ 

struct grafwindow *dw;			/* for data window */
struct grafwindow *tw;			/* for title window */
struct grafwindow *xw;			/* for x axis label window */
struct grafwindow *yw;			/* for y axis label window */
struct grafwindow *iw;			/* for information window */
struct grafwindow *pw;			/* for parameter window */

int	autoscale = TRUE;		/* autoscale or not */

unsigned mouse;				/* mouse present flag */

int	cxpos, cypos;			/* current cursor position */

int 	prtyp = NULLPT;			/* printer type */

int	lastnum = 0;			/* the last number of bins - used to
					   see if the current parameters are
					   valid */

unsigned rseed;				/* random number generator seed */

char	snumber[7] = "HE00000";		/* serial number */
char	author[13] = "Mark E. Nash";	/* author */

/* distribution parameters */
double params[MAXPARS+1][NUMDIST+1];	/* allow for the null distribution 
					   and already fit flag */

/* menu definitions */

/* menu set title */
char menuset[] = "HELICON";

/* FILE menu - adjust FLSIZE when changing this global */
char *flist[] = 
	{
	"File",			/* menu title */
	"File Name...",		/* menu items */
	"Save",
	"Print",
	"Quit",
	};

/* ANALYZE menu - adjust ANASIZE when changing this global */
char *analist[] = 
	{
	"Analyze",			/* menu title */
	"Continuous",			/* menu items */
	"Discrete",
	"Compare",
	"Auto Scale",
	"Manual Scale...",
	"Constants...",
	};

/* FIT menu - adjust FITSIZE when changing this global */
char *fitlist[] = 
	{
	"Fit",			/* menu title */
	"Rectangular",		/* menu items */
	"Triangular",
	"Normal",
	"Gamma",
	"Pareto",
	"F",
	"Beta",
	"Lognormal",
	"Exponential",
	"Double Exponential",
	"Gumbel",
	"Chi-square",
	"Logistic",
	"t",
	"Weibull",
	"Binomial",
	"Poisson",
	"Geometric",
	};

/* BUILD menu - adjust CPSIZE when changing this global */
char *blist[] = 
	{
	"Build",		/* menu title */
	"Estimate...",		/* menu items */
	};
	
/* DISPLAY menu - adjust DSPSIZE when changing this global */
char *dsplist[] =
	{
	"Display",		/* menu title */
	"Titles...",		/* menu items */
	};

/* UTILITY menu - adjust UTISIZE when changing this global */
char *utilist[] = 
	{
	"Utilities",			/* menu title */
	"Printer type...",		/* menu items */
	"Helicon info...",
	};


/* dialogs */
/* dialog macros that are only used in this file */
#define	D1X1	195+VPIXCHAR/2+11*VPIXCHAR	/* dialog box 1 eboxes ulc */
#define	DL2X1	(639-HPIXCHAR*(16+TICOLS+1))/2-1/* dialog box 2 ulc */
#define	D2X2	DL2X1+HPIXCHAR*16		/* dialog box 2 eboxes ulc */
#define	DL2SIZE	18+TICOLS			/* char col size of dialog 2 */
#define	DL3X1	243				/* dialog box 3 ulc */
#define	DL4X1	0				/* ulc's of dialogs 4,5,6 */
#define	DL4Y1	18
#define	DL5X1	DL4X1
#define DL5Y1	DL4Y1
#define	DL6X1	DL4X1
#define DL6Y1	DL4Y1
#define	DL4SIZE	30			/* number of cols for dialogs 4,5,6 */
#define	DL5SIZE	30
#define	DL6SIZE	67
#define	DL7SIZE	45	/* dialog 7 macros */
#define	DL7X1	((EGAPIXCOLS - HPIXCHAR*DL7SIZE) / 2)
#define	DL7Y1	100

/* dstr contains the ordered dialog text strings */
char *dstr[] =
	{
/* dialog 0 */
	"Enter input file name",
	"_ENDDIALOG",
/* dialog 1 */
	"Scale Data Window",
	"Minimum X:",
	"Maximum X:",
	"Minimum Y:",
	"Maximum Y:",
	"Number of Bins:",
	"_ENDDIALOG",
/* dialog 2 */
	"Graph Labels",
	"Graph Title:",
	"Graph Subtitle:",
	"X-Axis Label:",
	"Y-Axis Label:",
	"_ENDDIALOG",
/* dialog 3 */
	"Enter Printer Type",
	"_ENDDIALOG",
/* dialog 4 */
	"Enter the range of the points",
	"Minimum:",
	"Maximum:",
	"_ENDDIALOG",
/* dialog 5 */
	"Enter the median:",
	"_ENDDIALOG",
/* dialog 6 */
	"Enter the point which divides the range between the minimum and the",
	"median into two ranges of equal probability:",
	"Enter the point which divides the range between the median and the",
	"maximum into two ranges of equal probability:",
	"_ENDDIALOG",
/* dialog 7 */
	"User Adjustable Constants",
	"Random number seed (custom):",
	"Initial b-value (weibull):",
	"Successive error limit (weibull):",
	"_ENDDIALOG",
	};

/* dlocs is the location of the dialog strings in the dialog box.  Each pair
   of ints has a one-to-one correspondence with the string dstr. */
int dlocs[] =
	{
/* dialog 0 */
	0, 0,		/* (x,y) char coord in dialog box */
/* dialog 1 */
	7, 0,
	0, 2,
	0, 4,
	0, 6,
	0, 8,
	0, 12,
/* dialog 2 */
	(DL2SIZE)/2, 0,
	0, 2,
	0, 4,
	0, 6,
	0, 8,
/* dialog 3 */
	0, 0,
/* dialog 4 */
	0, 0,
	0, 2,
	0, 4,
/* dialog 5 */
	0, 0,
/* dialog 6 */
	0, 0,
	0, 2,
	0, 5,
	0, 7,
/* dialog 7 */
	9, 0,
	0, 2,
	0, 4,
	0, 6,
	};
	
/* einfo contains ordered edit box info */
int einfo[] =
	{
/* dialog 0 */
	  0,					/* ebox id */
	  239, ERRWY1+2*VPIXCHAR+VPIXCHAR/2,	/* ulc phys coords */
	  19,					/* char size of box */
	  DELIMITER,
	  
/* dialog 1 */
	  0,
	  D1X1, ERRWY1+2*VPIXCHAR,
	  SBOXSIZE,
	  1,
	  D1X1, ERRWY1+4*VPIXCHAR,
	  SBOXSIZE,
	  2,
	  D1X1, ERRWY1+6*VPIXCHAR,
	  SBOXSIZE,
	  3,
	  D1X1, ERRWY1+8*VPIXCHAR,
	  SBOXSIZE,
	  4,
	  195+VPIXCHAR/2+16*VPIXCHAR, ERRWY1+12*VPIXCHAR,
	  14,
	  DELIMITER,

/* dialog 2 */
	  0,
	  D2X2, ERRWY1+2*VPIXCHAR,
	  TICOLS+1,
	  1,
	  D2X2, ERRWY1+4*VPIXCHAR,
	  TICOLS+1,
	  2,
	  D2X2, ERRWY1+6*VPIXCHAR,
	  XACOLS+1,
	  3,
	  D2X2, ERRWY1+8*VPIXCHAR,
	  YAROWS+1,
	  DELIMITER,

/* dialog 3 */
	  0,					/* ebox id */
	  DL3X1+VPIXCHAR, ERRWY1+2*VPIXCHAR+VPIXCHAR/2,	/* ulc phys coords */
	  16,					/* char size of box */
	  DELIMITER,
	  
/* dialog 4 */
	  0,					/* ebox id */
/* ulc phys coords */
	  DL4X1+8*HPIXCHAR+HPIXCHAR/2, DL4Y1+2*VPIXCHAR,
	  21,					/* char size of box */

	  1,					/* ebox id */
/* ulc phys coords */
	  DL4X1+8*HPIXCHAR+HPIXCHAR/2, DL4Y1+4*VPIXCHAR,
	  21,					/* char size of box */
	  DELIMITER,

/* dialog 5 */
	  0,					/* ebox id */
	  DL5X1+17*HPIXCHAR+HPIXCHAR/2, DL5Y1+VPIXCHAR/2, /* ulc phys coords */
	  12,					/* char size of box */
	  DELIMITER,

/* dialog 6 */
	  0,					/* ebox id */
/* ulc phys coords */
	  DL6X1+44*HPIXCHAR+HPIXCHAR/2, DL6Y1+2*VPIXCHAR,
	  22,					/* char size of box */

	  1,					/* ebox id */
/* ulc phys coords */
	  DL6X1+45*HPIXCHAR+HPIXCHAR/2, DL6Y1+7*VPIXCHAR,
	  21,					/* char size of box */
	  DELIMITER,
	  
/* dialog 7 */
	  0,
	  DL7X1+28*HPIXCHAR+HPIXCHAR/2, DL7Y1+2*VPIXCHAR,
	  16,
	  1,
	  DL7X1+28*HPIXCHAR+HPIXCHAR/2, DL7Y1+4*VPIXCHAR,
	  16,
	  2,
	  DL7X1+33*HPIXCHAR+HPIXCHAR/2, DL7Y1+6*VPIXCHAR,
	  11,
	  DELIMITER,
	};
	
/* dlogs contains ordered dialog info */
int dlogs[] =
	{
	  231, ERRWY1,		/* upper left corner phys coord */
	  21, 4,		/* character size of dialog box */
	
	  195, ERRWY1,
	  31, 14,
	  
	  DL2X1, ERRWY1,
	  DL2SIZE, 10,
	  
	  DL3X1, ERRWY1,	/* upper left corner phys coord */
	  18, 4,		/* character size of dialog box */

	  DL4X1, DL4Y1,
	  DL4SIZE, 6,
	  
	  DL5X1, DL5Y1,
	  DL5SIZE, 2,
	  
	  DL6X1, DL6Y1,
	  DL6SIZE, 9,
	  
	  DL7X1, DL7Y1,
	  DL7SIZE, 8,
	  
	  ENDLOG
	};
	
/* dialog tbox table */
char	tboxes[NUMTBXS][TBOXSIZE+1];
int	tbxctr = 0;			/* counter for tboxes */

/* mouse globals */
unsigned int m1, m2, m3, m4;		/* parameters to mouse function */

/* for clearing strings on the graphics screen */
char	clrstr[81] = "\200\200\200\200\200\200\200\200\200\200\200\200\200\200\
\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\
\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\
\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\200\
\200\200\200\200\200\200\200\200\200";

/* table for gamma function values */
double gamtable[100] = { 
		       1.00000, .99433, .98884, .98355, .97844,
			.97350, .96874, .96415, .95973, .95546,
			.95135, .94739, .94359, .93993, .93642,
			.93304, .92980, .92670, .92373, .92088,
			.91817, .91558, .91311, .91075, .90852,
			
			.90640, .90440, .90250, .90072, .89904,
			.89747, .89600, .89464, .89338, .89222,
			.89115, .89018, .88931, .88854, .88785,
			.88726, .88676, .88636, .88604, .88580,
			.88565, .88560, .88563, .88575, .88595,
			
			.88623, .88659, .88704, .88757, .88818,
			.88887, .88964, .89049, .89142, .89243,
			.89352, .89468, .89592, .89724, .89864,
			.90012, .90167, .90330, .90500, .90678,
			.90864, .91057, .91258, .91466, .91683,
			
			.91906, .92137, .92376, .92623, .92877,
			.93138, .93408, .93685, .93969, .94261,
			.94561, .94869, .95184, .95507, .95838,
			.96177, .96523, .96878, .97240, .97610,
			.97988, .98374, .98768, .99171, .99581
			};
