
/*
   helimacs.h
   
   macros for helicon
   
   Author: Mark. E. Nash
   Date:   May, 1988
*/

/* boolean values */
#define		FALSE		0
#define		TRUE		0xFF

/* ega macros */
#define		EGAPIXCOLS	640		/* horizontal resolution */
#define		EGAPIXROWS	350		/* vertical resolution */
#define		ENABLE		0x0F		/* enables writing */
#define		INDEXREG	0x3CE		/* ega index register */
#define		VALREG		0x3CF		/* ega value register */

#define		OUTINDEX(index, val)	{ outp(INDEXREG, index);\
					  outp(VALREG, val); }
					  
#define		EGABASE		0xA0000000L	/* base memory of ega */
#define		EGABR		80		/* bytes per row */
#define		WIDTH		80L		/* character line */
#define		XMAX		EGAPIXCOLS-1	/* maximum x pixel */
#define		YMAX		EGAPIXROWS-1	/* maximum y pixel */
#define		XMIN		0		/* minimum x pixel */
#define		YMIN		0		/* minimum y pixel */

/* functions for pixel setting */
#define		XOR		0x18
#define		AND		0x08
#define		OR		0x10


/* general macros */
#define		RETURN		13	/* character code */

#define 	NUMDIST		18	/* Number of distribution functions */

#define		WHITESPACE	" \t\n"

#define		EGASPECT	0.762	/* aspect ratio of ega screen */

#define		DIVIDER		"--------"	/* divides entries of a file */

#define		FILNAMSIZ	80	/* maximum file name size */

#define		FORMFEED	"\014"	/* form-feed control character */

/* timer macros */
#define		DELMAX		32000	/* for finding timer constant */
#define		ZSPEED		3.00E+05	/* speed constant - increase
						   this number to increase
						   timer delays.  This
						   number is intimately
						   related to DELMAX */
#define		BLCMAX		6	/* since the time constant is
					   relatively short, this will
					   increase it for menu selection
					   blinking. */
#define		CBLMAX		12	/* blink delay for the dialog box
					   cursor. */
#define		BHIMAX		6	/* button selection hilight time */

#define		BIGSTRING	256	/* string for reading/writing files */
#define		MESSLEN		80		/* message length */
#define		LABELSIZE	20		/* label length */

#define		SCONST		.10	/* scale to 10% over extremes */
#define		BSCONST		.20	/* scale to 20% over extremes */

#define		NUMBINS		20		/* number of bins to use */
#define		BDNUMBINS	4	/* number of bins to use in c. dist */

#define		MSERR		-1.0		/* m sq err, ch-sq ret val */
#define		NOERR		TRUE		/* success return value */
#define		ERR		FALSE		/* fail return value */

/* distribution classes and plotting macros */
#define		CHOOSE		0
#define		DISCRETE	1
#define		CONTINUOUS	2


/****** GRAPHICS MACROS ******/
#define BLACK          0			/* colors */
#define BLUE           1
#define GREEN          2
#define CYAN           3
#define RED            4
#define MAGENTA        5
#define BROWN          6
#define WHITE          7
#define GRAY           8
#define LIGHTBLUE      9
#define LIGHTGREEN     10
#define LIGHTCYAN      11
#define LIGHTRED       12
#define LIGHTMAGENTA   13
#define LIGHTYELLOW    14
#define BRIGHTWHITE    15

#define		SOLID_LINE	0xFFFF		/* solid line mask */


/* pixels per character for ega graphics font */
#define		VPIXCHAR	8
#define		HPIXCHAR	8
#define		VLINSIZE	(VPIXCHAR+1)	/* vertical line size */

/* ega text */
#define		EMAXCOL		79
#define		EMAXROW		42
#define		EMINROW		0

/* tick mark multipliers - determines the length of a tick mark */
#define		MAJTIC		0.02		/* major tick */
#define		MINTIC		0.01		/* minor tick */


/******** MENU MACROS ********/
#define		MSTX		0	/* x and y phys coords of - */
#define		MSTY		4	/*    - menu set title */
#define		MENUERR		-1	/* error return for menu routines */
#define		MISPACE		2	/* char spaces between menu titles */
#define		BARSTART	1	/* column to start first menu at */
#define		BLCYCLE		8	/* select blink */

#define		TBX1		0	/* phys coord of title bar beg col */
#define		TBY1		0	/* phys coord of title bar beg row */
#define		TBX2		639	/* phys coord of title bar end col */
#define		TBY2		8	/* phys coord of title bar end row */

#define		MBX1		0	/* phys coord of menu bar beg col */
#define		MBY1		9	/* phys coord of menu bar beg row */
#define		MBX2		639	/* phys coord of menu bar end col */
#define		MBY2		17	/* phys coord of menu bar end row */

#define		TBCOLOR		BLUE		/* title bar color */
#define		TCOLOR		LIGHTRED	/* title color */
#define		MBCOLOR		GREEN		/* menu bar color */
#define		MBICOLOR	LIGHTYELLOW	/* menu bar item color */
#define		UBCOLOR		GREEN		/* menu item backgrnd color */
#define		UICOLOR		LIGHTYELLOW	/* menu item color */

#define		MAXMENUSIZE	42	/* maximum selections in menu */
#define		FLSIZE		5	/* size of FILE menu item list */
#define		ANASIZE		7	/* size of ANALYZE menu item list */
#define		FITSIZE		19	/* size of FIT menu item list */
#define		BSIZE		2	/* size of BUILD menu item list */
#define		DSPSIZE		2	/* size of DISPLAY menu item list */
#define		UTISIZE		3	/* size of UTILITY menu item list */

/* menu id codes */
#define		NULLID		-1	/* null menu or item id code */

#define		FILEID		0	/* menu id */
#define		FNID		0	/* item id's */
#define		SAVID		1
#define		PRID		2
#define		QID		3

#define		ANAID		1	/* menu id */
#define		CONTID		0	/* item id's */
#define		DISCID		1
#define		COMPID		2
#define		ASCALID		3
#define		MSCALID		4
#define		CONSTID		5

#define		FITID		2	/* menu id */
/* fit menu item id's are the index - 1 */

#define		BUILDID		3	/* menu id */
#define		ESTID		0	/* item id's */

#define		DSPID		4	/* menu id */
#define		TID		0	/* item id's */

#define		UTILID		5	/* menu id */
#define		PRTYID		0	/* item id's */
#define		HINFO		1


/****** WINDOW MACROS ******/

/* window types */
#define		GRAFWIN		0
#define		TEXTWIN		1
#define		BWIN		2

#define		WRLEN		80	/* maximum length of a window row */

/* window scale definitions */
#define		XAXIS		1
#define		YAXIS		2

/* error window physical coordinates */
#define		ERRWX1		112
#define		ERRWY1		100

/* error window scale coords */
#define		ERRCOLS		50
#define		ERROWS		3

/* error window scale coords */
#define		MSGCOLS		50
#define		MSGROWS		2

#define		ERRLEN		ERRCOLS		/* error message length */

/* error window prompt string coord. */
#define		EPCOL		0
#define		EPROW		2

/* data window physical coordinates */
#define		DATAWX1		282
#define		DATAWY1		48
#define		DATAWX2		639
#define		DATAWY2		325

/* title text window physical coordinates */
#define		TIWX1		DATAWX1
#define		TIWY1		DATAWY1-HPIXCHAR*3-1
#define		TICOLS		((DATAWX2-DATAWX1)/HPIXCHAR-1)
#define		TIROWS		2
/* x-axis label text window physical coordinates */
#define		XAWX1		DATAWX1
#define		XAWY1		DATAWY2+1
#define		XACOLS		((DATAWX2-DATAWX1)/HPIXCHAR-1)
#define		XAROWS		1
/* y-axis label text window physical coordinates */
#define		YACOLS		1
#define		YAWX1		(DATAWX1-(YACOLS+1)*HPIXCHAR-1)
#define		YAWY1		(MBY2+1)
#define		YAROWS		((349-MBY2+1)/VPIXCHAR-1)
/* information text window physical coordinates */
#define		IWX1		0
#define		IWY1		(MBY2+1)
#define		IWCOLS		(YAWX1/HPIXCHAR-1)
#define		IWROWS		8
/* parameter text window physical coordinates */
#define		PWX1		0
#define		PWY1		(IWY1+(IWROWS+1)*HPIXCHAR)
#define		PWCOLS		((YAWX1-1)/HPIXCHAR-1)
#define		PWROWS		((349-VPIXCHAR*11-PWY1)/VPIXCHAR-1)

/* compare window physical coordinates */
#define		CWCOLS		32
#define		CWROWS		NUMDIST+3
#define		CWX1		((EGAPIXCOLS-HPIXCHAR*CWCOLS)/2)
#define		CWY1		((EGAPIXROWS-VPIXCHAR*CWROWS)/2)

/* "about" window macros */
#define		ABCOLSIZE	34
#define		ABX1		((EGAPIXCOLS-HPIXCHAR*ABCOLSIZE)/2)
#define		ABY1		100
#define		ABBORCOLOR	BRIGHTWHITE
#define		ABBACKCOLOR	BLUE
#define		ABTICOLOR	LIGHTYELLOW
#define		ABINCOLOR	LIGHTRED

/* other window macros */
#define		IWFNCOL		14	/* start col of info window data */
#define		SPAROW		1	/* start row of param window data */

/* data window colors */
#define		DATABKCOLOR	BLACK		/* background color */
#define		DATABRCOLOR	RED		/* data window border color */
#define		AXISCOLOR	BROWN		/* zero lines */
#define		SCALECOLOR	CYAN		/* scales */
#define		SCLABCOLOR	CYAN		/* scale labels */
#define		SCACOLOR	GRAY		/* scatter point color */
#define		DPCOLOR		BRIGHTWHITE	/* data point color */
#define		FUNCOLOR	GREEN		/* function color */
#define		MEANCOLOR	BRIGHTWHITE	/* mean indicator */

/* text window colors */
#define		IBRCOLOR	BLACK		/* text wins' border color */
#define		IBKCOLOR	BLACK		/* text wins' backgr color */
#define		GLCOLOR		LIGHTYELLOW	/* graph label text color */
#define		IWSCOLOR	LIGHTCYAN	/* info window txt */
#define		IWDCOLOR	BRIGHTWHITE	/* info win data */
#define		PWSCOLOR	LIGHTYELLOW	/* parameter win txt */
#define		PWDCOLOR	BRIGHTWHITE	/* par win data */

/* compare window colors */
#define		CWBRCOLOR	BRIGHTWHITE
#define		CWBKCOLOR	LIGHTBLUE
#define		CWTCOLOR	LIGHTYELLOW
#define		CWDCOLOR	WHITE
#define		CWHCOLOR	BRIGHTWHITE


#define		ERRBORDER	BLUE		/* err window border color */
#define		ERRBACK		RED		/* err background color */
#define		EMESCOLOR	LIGHTYELLOW	/* error message color */
#define		EPRCOLOR	BRIGHTWHITE	/* prompt color */


#define		MSGBORDER	BRIGHTWHITE	/* msg window border color */
#define		MSGBACK		BLUE		/* msg background color */
#define		MESCOLOR	LIGHTYELLOW	/* message color */

/* calculation status window macros (weibull calculations) */
/* char size of button win */
#define		CAWROWS		(MSGROWS+2)
#define		CBWX1	(ERRWX1+HPIXCHAR*(MSGCOLS-CAWROWS-3))
#define		CBWY1	(ERRWY1+VPIXCHAR*(CAWROWS-1)-VPIXCHAR/2)

/* calculation status info (weibull calculation) parameter area */
#define		CSCOL		4
#define		CSROW		3
/* size of area to display parameters */
#define		CSSIZE		(MSGCOLS-CAWROWS-8)

/* clipping macros */
#define		PLOT		1
#define		NOPLOT		0


/******** distribution macros ********/
#define		NUMDIST		18		/* number of distributions */
#define		MAXPARS		5	/* max # of parameters for any dist.,
					   plus the error value. */
#define		CSECT		4	/* sections in a custom dist. */
#define		CPTS		250	/* points per section in c. dist. */

#define		CHKCNT		100	/* check for cancel button (weibull) */

/* index into the params array */
#define		NULLDIST	0

#define		RECT		1

#define		TRI		2
#define		MPAR		3	/* parameter - x at maximum f(x) */

#define		NOR		3	/* distribution number */
#define		NMEAN		1	/* parameter - mu or mean */
#define		NSDEV		2	/* parameter - sigma or stan. dev */

#define		GAM		4
#define		CPAR		1	/* parameter - C */
#define		RPAR		2	/* parameter - r */

#define		PAR		5

#define		FDIST		6
#define		FMPAR		1	/* parameter m */
#define		FNPAR		2	/* parameter n */

#define		BETA		7

#define		LOGNOR		8
#define		LMPAR		1	/* parameter m */
#define		LRPAR		2	/* parameter r */

#define		EXPON		9

#define		DEXPON		10

#define		GUM		11

#define		CHI		12

#define		LOGIS		13

#define		TDIST		14

#define		WEI		15		/* distribution number */

#define		BINOM		16
#define		PPAR		1		/* parameter p index */
#define		NPAR		2		/* parameter n index */

#define		POI		17
#define		CPAR		1

#define		GEO		18

/* generic distribution parameters */
#define		MAG		0		/* magnitude parameter index 
						   for all distributions */
#define		APAR		1		/* parameter a index */
#define		BPAR		2		/* parameter b index */
#define		KPAR		1		/* parameter k */

#define		MSE		4		/* mse for a set of pars */

#define		FITQ		MAXPARS		/* already fit flag index
						   for all distributions */
						   
#define		FIRSTDISC	16		/* first discrete dist. */

/* possible fit flag values */
#define		DOFIT		0.0		/* calculate fit */
#define		NODOFIT		-1.0		/* already fit */



/****** Math Constants ******/
#define PI   	3.14159265358979323846


/* ega_font special characters */
#define		FBLOCK		"\200"		/* 128th character in font */


/******* EVENT MACROS *******/
#define		NULLEVENT	0		/* null event */
#define		CURUPDATE	1		/* cursor position update */
#define		ALTEVENT	2		/* alt key event */
#define		SELECT		3		/* select event */
#define		ERREVENT	-1		/* error event */

/******* KEY DEFINITIONS *******/
#define		OUTMENU		-1
#define		UP		72
#define		DOWN		80
#define		LEFT		75
#define		RIGHT		77
#define		DEL		83
#define		HOME		71
#define		ALT		50
#define		BACKSPACE	8
#define		ESC		27
#define		NULLKEY		255
#define		UPKEY		1
#define		DOWNKEY		2
#define		LEFTKEY		3
#define		RIGHTKEY	4
#define		DELKEY		7


/******* DIALOG MACROS *******/
#define		DELIMITER	-2	/* delimiter between einfo's */
#define		ENDLOG		-1	/* last dialog indicator */
#define		TBOXSIZE	80	/* dialog tbox size */
#define		SBOXSIZE	19	/* scale ebox size */

/* dialog id's */
#define		IFNDID		0	/* input file name dialog id */
#define		SCADID		1	/* scale dialog id */
#define		TITLID		2	/* titles dialog */
#define		PTYDID		3	/* printer type dialog */

#define		BMINMAX		4	/* dist. building dialogs */
#define		BMEDD		5
#define		BSUBMED		6

#define		CDID		7	/* user-defined constants dialog */

/* tbox array indices  - increase NUMTBXS when adding to this list */
#define		DATFIL		0	/* data (input) file name */

#define		TMINX		1	/* default scale values */
#define		TMAXX		2
#define		TMINY		3
#define		TMAXY		4
#define		NBINS		5	/* default number of bins */

#define		GTITL		6	/* graph title */
#define		GSTITL		7	/* graph sub-title */
#define		GXAXIS		8	/* graph x-axis label */
#define		GYAXIS		9	/* graph y-axis label */

#define		PRTYPE		10	/* printer type */

#define		BMIN		11	/* build min */
#define		BMAX		12	/* build max */
#define		BMED		13	/* build median */
#define		BLOW		14	/* low range point */
#define		BHI		15	/* high range point */

#define		SEED		16	/* random number seed */
#define		BVLU		17	/* initial b-value in weibull calc. */
#define		EVLU		18	/* weibull calc error limit */

#define		NUMTBXS		19	/* total number of tboxes */

/* dialog colors */
#define		DLBRCOLOR	LIGHTMAGENTA	/* dialog box border color */
#define		DLBKCOLOR	GRAY		/* dialog box back color */
#define		DLTXTCOLOR	LIGHTYELLOW	/* dialog box text color */
#define		EBBRCOLOR	WHITE		/* ebox border color */
#define		EBBKCOLOR	BLACK		/* ebox background color */
#define		EBTXCOLOR	BRIGHTWHITE	/* ebox text color */


/******* MEMORY MONITOR *******/
#define		ALLOC		0		/* allocate function */
#define		FRE		1		/* free function */
#define		MAXINT		65535		/* largest unsigned integer */

#define		MEMBRCOLOR	BRIGHTWHITE	/* mem mon border color */
#define		MEMBKCOLOR	BLUE		/* mem mon background color */
#define		MSTRCOLOR	LIGHTYELLOW	/* mem mon string color */
#define		MFCOLOR		BRIGHTWHITE	/* free memory indicator */
#define		MALCOLOR	LIGHTRED	/* allocated mem indicator */
#define		MOPCOLOR	LIGHTGREEN	/* op sys mem indicator */
#define		MTMCOLOR	LIGHTCYAN	/* total mem indicator */
#define		MSCACOLOR	CYAN		/* scale color */
#define		MEMDRCOLOR	WHITE		/* data win border */
#define		MEMDKCOLOR	BLACK		/* data win background */

#define		MEMTWX1		0		/* x,y of main window */
#define		MEMTWY1		260
#define		MEMDWX1		(HPIXCHAR/2+11*HPIXCHAR)/* ulc of data win */
#define		MEMDWY1		(MEMTWY1+7*VPIXCHAR)
#define		MEMDWX2		(MEMDWX1+20*HPIXCHAR)	/* lrc of data win */
#define		MEMDWY2		(MEMDWY1+VPIXCHAR/2+3*VPIXCHAR)

#define		MEMRY1		1.7		/* memory bar rect bounds */
#define		MEMRY2		2.5


/* mouse macros */
#define		MOUSEINT	0x33		/* mouse software int num */
#define		MOUSE(M1, M2, M3, M4)	{ m1 = M1; \
					  m2 = M2; \
					  m3 = M3; \
					  m4 = M4; \
					  fmouse(); }


/* printer macros */
/* printer types */
#define		NULLPT		0
#define		HPLASJET	1
#define		EPSON		2

/* printer types as strings */
#define		HPSUP		"hplasjet"
#define		EPSUP		"epson"

/* printer windows */
#define		SWCOLS		33		/* char size of stat window */
#define		SWROWS		5
/* ulc of status window */
#define		SWX1		((640-HPIXCHAR*(SWCOLS+1))/2)
#define		SWY1		ERRWY1
/* other stat window macros */
#define		PSWUC		29	/* text percent char col */
#define		PSWUR		0	/* text percent char row */

/* char size of button win */
#define		SBWCOLS		6
#define		SBWROWS		1
/* stat button win ulc */
#define		SBWX1	(SWX1+HPIXCHAR*(SWCOLS-SBWCOLS-1))
#define		SBWY1	(SWY1+VPIXCHAR*(SWROWS-SBWROWS-1)-VPIXCHAR/4)

/* stat graphics window ulc */
#define		SGWX1		(SWX1+HPIXCHAR)
#define		SGWY1		(SWY1+VPIXCHAR*(SWROWS-3))
/* stat graphics window lrc */
#define		SGWX2		(SBWX1-HPIXCHAR)
#define		SGWY2		(SGWY1+VPIXCHAR/2+3*VPIXCHAR)
/* other stat graf window macros */
#define		PRIY1		1.7		/* printer bar rect bounds */
#define		PRIY2		2.5


/* print window colors */
/* printer status window */
#define		STBRCOLOR	BRIGHTWHITE
#define		STBKCOLOR	BLUE
#define		PSSCOLOR	LIGHTYELLOW

/* printer status graphics window */
#define		STGBRCOLOR	BRIGHTWHITE
#define		STGBKCOLOR	BLACK
#define		PGSCACOLOR	CYAN
#define		STGDCOLOR	BRIGHTWHITE

/* printer status button window */
#define		SBBRCOLOR	BRIGHTWHITE
#define		SBBKCOLOR	RED
#define		PBSCOLOR	LIGHTYELLOW
#define		BHILCOLOR	BRIGHTWHITE


