
/*
   helmath2.c
   
   Distribution parameter calculation routines for helicon.
   
   Author: Mark E. Nash
   Date:   August, 1988
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<malloc.h>
#include	<math.h>
#include	<string.h>
#include	<conio.h>
#include	<graph.h>
#include	"helimacs.h"
#include	"helimess.h"
#include	"helstruc.h"
#include	"helfdef.h"


/* externals */
extern char msg[];				/* in heliglob.h */
extern char *fitlist[];				/* in heliglob.h */
extern struct rawcell *rd_top;			/* in heliglob.h */
extern struct menutop *menus;			/* in heliglob.h */
extern short cxpos, cypos;			/* in heliglob.h */
extern unsigned mouse;				/* in heliglob.h */
extern unsigned m1, m2, m3, m4;			/* in heliglob.h */
extern char clrstr[];				/* in heliglob.h */


int rectcalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   rectcalc() calculates the parameters of the rectangular distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	struct grafwindow *ms;		/* for messages */
	double	mag;			/* distribution parameters */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][RECT] == NODOFIT)
		return(NOERR);
		
/* get the parameters */
	parray[APAR][RECT] = btop->mean - sqrt(3.0*btop->var);
	parray[BPAR][RECT] = btop->mean + sqrt(3.0*btop->var);
		
/* get the magnitude - must be calculated last */
	if (magcalc(btop->start, RECT, parray, &mag) == ERR)
		return(ERR);
	parray[MAG][RECT] = mag;
		
/* update the fit flag */
	parray[FITQ][RECT] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, RECT-1);

	return(NOERR);
	}
	
	
	
int tricalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   tricalc() calculates the parameters of the triangular distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	struct grafwindow *ms;		/* for messages */
	struct bin *bp;			/* point to a bin */
	double	m, mag;			/* distribution parameters */
	int	found;			/* search for maximum */
	double	u, v;			/* for calculations */
	double	f1, f2, f3, f4;		/* for calculations */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][TRI] == NODOFIT)
		return(NOERR);
		
/* get the m parameter by finding where x is maximum */
	bp = btop->start;
	found = FALSE;
	while (bp != NULL && !found)
		if (bp->y == btop->ymax)
			{
			found = TRUE;
			m = bp->x;
			parray[MPAR][TRI] = m;
			}
		else
			bp = bp->next;
			
	if (!found)
		{
		sprintf(msg, D_NOMAX, btop->ymax, "tricalc()");
		err_window(msg);
		return(ERR);
		}
		
/* calculate a and b parameters */
	u = btop->mean;
	v = btop->var;
	f2 = m - 3.0 * u;
	f3 = -6.0*v + (pow(m, 2.0) + m*f2 + pow(f2, 2.0)) / 3.0;
	f4 = sqrt(pow(f2, 2.0) - 4.0 * f3);

	parray[BPAR][TRI] = 0.5 * (-f2 + f4);
	parray[APAR][TRI] = 3.0*u - m - parray[BPAR][TRI];
		
/* get the magnitude - must be calculated last */
	if (magcalc(btop->start, TRI, parray, &mag) == ERR)
		return(ERR);
	parray[MAG][TRI] = mag;
		
/* update the fit flag */
	parray[FITQ][TRI] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, TRI-1);

	return(NOERR);
	}
	
	
	
int norcalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   norcalc() calculates the parameters of the normal distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	double	mag;			/* distribution parameters */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][NOR] == NODOFIT)
		return(NOERR);
		
/* get the mean and standard dev */
	parray[NMEAN][NOR] = btop->mean;
	parray[NSDEV][NOR] = sqrt(btop->var);
		
/* get the magnitude - must be calculated last */
	if (magcalc(btop->start, NOR, parray, &mag) == ERR)
		return(ERR);
	parray[MAG][NOR] = mag;
		
/* update the fit flag */
	parray[FITQ][NOR] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, NOR-1);

	return(NOERR);
	}
	
	
	
int gamcalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   gamcalc() calculates the parameters of the gamma distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	struct grafwindow *ms;		/* for messages */
	double	mag;			/* distribution parameters */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][GAM] == NODOFIT)
		return(NOERR);
		
/* get the parameters */
	parray[CPAR][GAM] = btop->mean / btop->var;
	parray[RPAR][GAM] = pow(btop->mean, 2.0) / btop->var;
	
	if (parray[CPAR][GAM] > 0.0 && parray[RPAR][GAM] > 0.0)
		{
/* get the magnitude - must be calculated last */
		if (magcalc(btop->start, GAM, parray, &mag) == ERR)
			return(ERR);
		parray[MAG][GAM] = mag;
		}
		
/* update the fit flag */
	parray[FITQ][GAM] = NODOFIT;

/* toggle the menu item hilight */
	toghil(menus, FITID, GAM-1);
	
	return(NOERR);
	}
	
	
	
int parcalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   parcalc() calculates the parameters of the pareto distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	double	mag, a, b, k;		/* distribution calculations */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][PAR] == NODOFIT)
		return(NOERR);
		
/* get the parameters */
	k = sqrt(1.0 + pow(btop->mean, 2.0) / btop->var);
	a = 1.0 + k;
	b = btop->mean * k / (1.0 + k);
	parray[APAR][PAR] = a;
	parray[BPAR][PAR] = b;
		
/* get the magnitude - must be calculated last */
	if (a > 2.0 && b > 1.0)
		{
		if (magcalc(btop->start, PAR, parray, &mag) == ERR)
			return(ERR);
		parray[MAG][PAR] = mag;
		}
		
/* update the fit flag */
	parray[FITQ][PAR] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, PAR-1);

	return(NOERR);
	}
	
	
	
int fcalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   fcalc() calculates the parameters of the F distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	struct grafwindow *ms;		/* for messages */
	double	mag, n, m, u, v;	/* distribution calculations */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][FDIST] == NODOFIT)
		return(NOERR);
		
/* get the parameters */
	u = btop->mean;
	v = btop->var;
	n = (double)((int)(2.0 * u / (u-1.0) + 0.5));
	m = (double)((int)((2.0 * pow(n, 2.0) * (n-2.0)) / (v * 
			pow((n-2.0), 2.0) * (n-4.0) - 2.0*pow(n, 2.0)) + 0.5));
	
	parray[FMPAR][FDIST] = m;
	parray[FNPAR][FDIST] = n;
		
/* get the magnitude - must be calculated last */
	if (m > 0.0 && n > 0.0)
		{
		if (magcalc(btop->start, FDIST, parray, &mag) == ERR)
			return(ERR);
		parray[MAG][FDIST] = mag;
		}
		
/* update the fit flag */
	parray[FITQ][FDIST] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, FDIST-1);

	return(NOERR);
	}
	
	
	
int betacalc(btop, parray)
	struct topbin *btop;
	double parray[MAXPARS+1][NUMDIST+1];
/*
   betacalc() calculates the parameters of the beta distribution.
   
   Entry: btop points to the top of the bin list.
          rtop points to the top of the raw data list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	struct grafwindow *ms;		/* for messages */
	double	a, b, mag;		/* statistical parameters */
	double	k;			/* for calculations */


	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][BETA] == NODOFIT)
		return(NOERR);
		
/* first find k */
	k = 1.0 / btop->mean - 1.0;
	
/* now get a and b */
	a = (k/btop->var - pow(k+1.0, 2.0)) / pow(k+1.0, 3.0);
	b = (pow(k, 2.0)/btop->var - k * (k+1.0)) / pow(k+1.0, 3.0);
	parray[APAR][BETA] = a;
	parray[BPAR][BETA] = b; 
	
/* get the magnitude - must be calculated last */
	if (a > 0.0 && b > 0.0)
		{
		if (magcalc(btop->start, BETA, parray, &mag) == ERR)
			return(ERR);
		parray[MAG][BETA] = mag;
		}
		
/* update the fit flag */
	parray[FITQ][BETA] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, BETA-1);

	return(NOERR);
	}
	
	
	
int lnorcalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   lnorcalc() calculates the parameters of the lognormal distribution.
   
   Entry: btop points to the top of the bin list.
          rtop points to the top of the raw data list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	struct grafwindow *ms;		/* for messages */
	struct rawcell *rtop;		/* top of raw data list */
	struct rawcell *rp;		/* for stepping through the list */
	double	mag;			/* statistical parameters */
	double	m, r;			/* distribution parameters */
	double	n, s1, s2;		/* variables for calculation */


	rtop = rd_top;
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][LOGNOR] == NODOFIT)
		return(NOERR);
		
/* display calculation message */
	sprintf(msg, I_CALC, fitlist[LOGNOR]);
	if ((ms = msg_window(msg)) == NULL)
		{
		printf("\7\7\7");
		return(ERR);
		}
		
/* calculate the parameters */
/* first find n */
	n = 0.0;
	for (rp=rtop; rp!=NULL; rp=rp->next)
		n += 1.0;

/* now find the sums */
	s1 = 0.0;
	for (rp=rtop; rp!=NULL; rp=rp->next)
		if (rp->x <= 0.0);	/* ignore this point */
		else
			s1 += log(rp->x);
			
	m = s1 * 1.0/n;

	s2 = 0.0;
	for (rp=rtop; rp!=NULL; rp=rp->next)
		if (rp->x > 0.0)
			s2 += pow(log(rp->x) - m, 2.0);
		else;		/* ignore these points */
			
	r = sqrt(s2 * 1.0 / n);

/* put parameters in the parameter array */
	parray[LMPAR][LOGNOR] = m;
	parray[LRPAR][LOGNOR] = r;
	
/* get the magnitude - must be calculated last */
	if (magcalc(btop->start, LOGNOR, parray, &mag) == ERR)
		{
		close_grafwin(&ms);
		return(ERR);
		}
		
	parray[MAG][LOGNOR] = mag;
		
/* update the fit flag */
	parray[FITQ][LOGNOR] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, LOGNOR-1);

	close_grafwin(&ms);
	return(NOERR);
	}
	
	
	
int excalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   excalc() calculates the parameters of the exponential distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	double	mag;			/* distribution calculations */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][EXPON] == NODOFIT)
		return(NOERR);
		
/* get the parameters */
	parray[CPAR][EXPON] = 1.0 / btop->mean;
		
/* get the magnitude - must be calculated last */
	if (parray[CPAR][EXPON] > 0.0)
		{
		if (magcalc(btop->start, EXPON, parray, &mag) == ERR)
			return(ERR);
		parray[MAG][EXPON] = mag;
		}
		
/* update the fit flag */
	parray[FITQ][EXPON] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, EXPON-1);

	return(NOERR);
	}
	
	
	
int dexcalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   dexcalc() calculates the parameters of the double exponential dist.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	double	mag;			/* distribution calculations */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][DEXPON] == NODOFIT)
		return(NOERR);
		
/* get the parameters */
	parray[APAR][DEXPON] = btop->mean;
	parray[BPAR][DEXPON] = sqrt(2.0/btop->var);
		
/* get the magnitude - must be calculated last */
	if (magcalc(btop->start, DEXPON, parray, &mag) == ERR)
		return(ERR);
	parray[MAG][DEXPON] = mag;
		
/* update the fit flag */
	parray[FITQ][DEXPON] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, DEXPON-1);

	return(NOERR);
	}
	
	
	
int gumcalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   gumcalc() calculates the parameters of the Gumbel cumulative dist.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	double	a, b, mag;		/* distribution calculations */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][GUM] == NODOFIT)
		return(NOERR);
		
/* get the parameters */
	a = btop->mean - 0.577216 * sqrt(6.0*btop->var) / PI;
	b = sqrt(6.0*btop->var) / PI;
	
	parray[APAR][GUM] = a;
	parray[BPAR][GUM] = b;
		
/* get the magnitude - must be calculated last */
	if (magcalc(btop->start, GUM, parray, &mag) == ERR)
		return(ERR);
	parray[MAG][GUM] = mag;
		
/* update the fit flag */
	parray[FITQ][GUM] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, GUM-1);

	return(NOERR);
	}
	
	
	
int chicalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   chicalc() calculates the parameters of the chi-square distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	double	mag, k;			/* distribution calculations */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][CHI] == NODOFIT)
		return(NOERR);
		
/* get the parameters */
	k = btop->mean;
	parray[KPAR][CHI] = k;
		
/* get the magnitude - must be calculated last */
	if (k > 0.0)
		{
		if (magcalc(btop->start, CHI, parray, &mag) == ERR)
			return(ERR);
		parray[MAG][CHI] = mag;
		}
		
/* update the fit flag */
	parray[FITQ][CHI] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, CHI-1);

	return(NOERR);
	}
	
	
	
int logcalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   logcalc() calculates the parameters of the logistic cumulative distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	double	mag, a, b;		/* distribution calculations */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][LOGIS] == NODOFIT)
		return(NOERR);
		
/* get the parameters */
	a = btop->mean;
	b = sqrt(3.0*btop->var) / PI;
	parray[APAR][LOGIS] = a;
	parray[BPAR][LOGIS] = b;
		
/* get the magnitude - must be calculated last */
	if (magcalc(btop->start, LOGIS, parray, &mag) == ERR)
		return(ERR);
	parray[MAG][LOGIS] = mag;
		
/* update the fit flag */
	parray[FITQ][LOGIS] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, LOGIS-1);

	return(NOERR);
	}
	
	
	
int tcalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   tcalc() calculates the parameters of the t distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	double	mag, k;			/* distribution calculations */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][TDIST] == NODOFIT)
		return(NOERR);
		
/* get the parameters */
	k = 2.0 * btop->var / (btop->var - 1);
	parray[KPAR][TDIST] = k;
		
/* get the magnitude - must be calculated last */
	if (k > 2.0)
		{
		if (magcalc(btop->start, TDIST, parray, &mag) == ERR)
			return(ERR);
		parray[MAG][TDIST] = mag;
		}
		
/* update the fit flag */
	parray[FITQ][TDIST] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, TDIST-1);

	return(NOERR);
	}
	
	
	
int weicalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   weicalc() calculates the parameters of the weibull distribution.
   
   Entry: btop points to the top of the bin list.
          rtop points to the top of the raw data list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	struct grafwindow *ms, *pb;	/* for messages and status */
	struct rawcell *rtop;		/* top of raw data list */
	struct rawcell *rp;		/* for stepping through the list */
	double	mag;			/* statistical parameters */
	double	a, b;			/* distribution parameters */
	double	b1, n, lx, bx, s1, s2, s3;	/* variables for calculation */
	struct xycoord xy;		/* coord conv */
	int	done, ctr;		/* calculation cancellation vars */
	int	key;			/* for cancel button */
	char	tempchr;		/* temp storage for clear string */
	extern char tboxes[NUMTBXS][TBOXSIZE+1];	/* in heliglob.h */


	rtop = rd_top;
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][WEI] == NODOFIT)
		return(NOERR);
		
/* display calculation message and status */
	if ((ms = open_grafwin(TEXTWIN, _getlogcoord(ERRWX1, ERRWY1).xcoord,
		                       _getlogcoord(ERRWX1, ERRWY1).ycoord,
			   0, 0, MSGBORDER, MSGBACK, MSGCOLS, CAWROWS, TRUE))
								== NULL)
		return(ERR);

/* open the button window */
	xy = _getlogcoord(CBWX1, CBWY1);
	if ((pb = open_grafwin(BWIN, xy.xcoord, xy.ycoord, 0, 0, SBBRCOLOR, 
				SBBKCOLOR, SBWCOLS, SBWROWS, TRUE)) == NULL)
		return(ERR);

/* display the window information */
	sprintf(msg, I_CALC, fitlist[WEI]);
	eputs_window(ms, msg, MESCOLOR, TRUE);
	eputs_window(pb, "Cancel", PBSCOLOR, FALSE);
		
/* calculate the b parameter */
/* first find n */
	n = 0.0;
	for (rp=rtop; rp!=NULL; rp=rp->next)
		n += 1.0;

/* now find the sums */
	b = atof(tboxes[BVLU]);		/* initial guess for b */
	s1 = 0.0;
	s2 = 0.0;
	s3 = 0.0;
	for (rp=rtop; rp!=NULL; rp=rp->next)
		{
		if (rp->x <= 0.0);	/* ignore this point */
		else
			{
			lx = log(rp->x);
			bx = pow(rp->x, b);
			s1 += bx * lx;
			s2 += bx;
			s3 += lx;
			}
		}
	b1 = b;
	b = n / ((n*s1/s2) - s3);

/* update the status of the calculation */
	gtwcurpos(ms, CSCOL, CSROW);
	tempchr = clrstr[CSSIZE];
	clrstr[CSSIZE] = '\0';
	eputs_window(ms, clrstr, MSGBACK, FALSE);
	clrstr[CSSIZE] = tempchr;
	gtwcurpos(ms, CSCOL-4, CSROW);
	sprintf(msg, "b = %-g", b);
	eputs_window(ms, msg, PWDCOLOR, FALSE);

/* iterate while error is larger than error */
	done = FALSE;
	ctr = 0;
	while (fabs(b-b1) > atof(tboxes[EVLU]) && !done)
		{
		s1 = 0.0;
		s2 = 0.0;
		for (rp=rtop; rp!=NULL; rp=rp->next)
			{
			if (rp->x > 0.0)
				{
				bx = pow(rp->x, b);
				s1 += bx * log(rp->x);
				s2 += bx;	/* s3 always the same */
				if (ctr++ == CHKCNT)
					{
					ctr = 0;	/* reset counter */
		/* check for cancel button */
			/* get mouse info */
					MOUSE(3, 0, 0, 0);
					xy = _getlogcoord(m3, m4);
					cxpos = xy.xcoord;
					cypos = xy.ycoord;
			/* get key info */
					if (kbhit())
						key = getch();
			/* check key and mouse */
					if (key == ESC || 
					     (curnrgn(&(pb->wbox)) && m2 != 0))
						{
						hilbtn(pb);
						close_grafwin(&pb);
						done = TRUE;
						}
					}
				}
			}
		b1 = b;
		b = n / ((n*s1/s2) - s3);
/* update the status of the calculation */
		gtwcurpos(ms, CSCOL, CSROW);
		tempchr = clrstr[CSSIZE];
		clrstr[CSSIZE] = '\0';
		eputs_window(ms, clrstr, MSGBACK, FALSE);
		clrstr[CSSIZE] = tempchr;
		gtwcurpos(ms, CSCOL-4, CSROW);
		sprintf(msg, "b = %-g", b);
		eputs_window(ms, msg, PWDCOLOR, FALSE);
		}
		
/* now calculate a */
	s2 = 0.0;
	for (rp=rtop; rp!=NULL; rp=rp->next)
		if (rp->x > 0.0)
			s2 += pow(rp->x, b);

	a = n / s2;

/* put parameters in the parameter array */
	parray[APAR][WEI] = a;
	parray[BPAR][WEI] = b;
	
/* get the magnitude - must be calculated last */
	if (magcalc(btop->start, WEI, parray, &mag) == ERR)
		return(ERR);
	parray[MAG][WEI] = mag;
		
/* update the fit flag */
	parray[FITQ][WEI] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, WEI-1);

	close_grafwin(&pb);
	close_grafwin(&ms);
	return(NOERR);
	}
	
	
	
int bincalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   bincalc() calculates the parameters of the binomial distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	double	mag, p, n;		/* distribution parameters */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][BINOM] == NODOFIT)
		return(NOERR);

	p = 1.0 - btop->var / btop->mean;
	n = (double)((int)(btop->mean / p + 0.5));
	parray[PPAR][BINOM] = p;
	parray[NPAR][BINOM] = n;
		
/* get the magnitude - must be calculated last */
	if (p > 0.0 && p < 1.0 && n > 0.0)
		{
		if (magcalc(btop->start, BINOM, parray, &mag) == ERR)
			return(ERR);
		parray[MAG][BINOM] = mag;
		}

/* update the fit flag */
	parray[FITQ][BINOM] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, BINOM-1);

	return(NOERR);
	}
	
	

int poicalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   poicalc() calculates the parameters of the poisson distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	double c, mag;			/* distribution parameters */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][POI] == NODOFIT)
		return(NOERR);
		
/* get the mean */
	c = btop->mean;
	parray[CPAR][POI] = c;
		
/* get the magnitude - must be calculated last */
	if (c > 0.0)
		{
		if (magcalc(btop->start, POI, parray, &mag) == ERR)
			return(ERR);
		parray[MAG][POI] = mag;
		}

/* update the fit flag */
	parray[FITQ][POI] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, POI-1);

	return(NOERR);
	}



int geocalc(btop, parray)
	struct topbin *btop;
	double	parray[MAXPARS+1][NUMDIST+1];
/*
   geocalc() calculates the parameters of the geometric distribution.
   
   Entry: btop points to the top of the bin list - the data to fit.
          parray is a reference to the parameter array.
	  
   Exit:  If no errors, the proper parameters in parray have been set,
		and returns NOERR,
          Else, return ERR.
*/
	{
	double p, q, mag;			/* distribution parameters */
	
	
	if (btop == NULL)
		{
		err_window(D_NOBINS);
		return(ERR);
		}

/* get out if calculations have already been performed */
	if (parray[FITQ][GEO] == NODOFIT)
		return(NOERR);
		
/* get the parameters */
	p = btop->mean / btop->var;
	parray[PPAR][GEO] = p;
	q = 1.0 - p;
		
/* get the magnitude - must be calculated last */
	if (p < 1.0)
		{
		if (magcalc(btop->start, GEO, parray, &mag) == ERR)
			return(ERR);
		parray[MAG][GEO] = mag;
		}

/* update the fit flag */
	parray[FITQ][GEO] = NODOFIT;
	
/* toggle the menu item hilight */
	toghil(menus, FITID, GEO-1);

	return(NOERR);
	}
