
/*
   helstruc.h
   
   Data structures used in Helicon.
   
   Author: Mark E. Nash
   Date:   June, 1988
*/

/* menu hilight area */
struct hibox
	{
	int x1, y1;			/* upper left corner of area */
	int x2, y2;			/* lower right corner of area */
	};

/* graphics window */
struct grafwindow
	{
	struct grafwindow *next_window;		/* next window in the list */
	int	list;				/* boolean to tell whether this
						   window is in the list 
						   (TRUE), or independent */
	unsigned char wintype;			/* graphics, text, or button */
	struct hibox wbox;			/* logical coords. of window,
	                                	   does not include pixel wide
						   borders, unless this is a
						   text window - this is the 
						   drawing area. */
	double	xsmin, ysmin, xsmax, ysmax;	/* x,y scale of window */
	double	npx, npy;			/* number per pixel */
	unsigned short maxcols, maxrows;/* char coords maxima of text window */
	unsigned short currow, curcol;	/* current cursor char pos for text */
	long 	size;			/* window size in bytes */
	short int ix1, iy1;		/* upper left corner of background */
	char far *background;		/* background image, NULL if none */
	};

	
/* menu structures */

/* top menu information */
struct menutop
	{
	char	*title;		/* menu set title */
	short	tbcolor;	/* title bar background color */
	short	tcolor;		/* title color */
	unsigned maxcols;	/* maximum number of useable menu columns */
	unsigned maxrows;	/* maximum number of useable menu rows */
	short	barcolor;	/* menu bar color */
	short	itemcolor;	/* color of menu items */
	struct menu *start;	/* start of menu list */
	};
	
/* individual menu starting point */
struct menu
	{
	char	*title;			/* title of this menu */
	unsigned pd;			/* TRUE if menu is pulled down */
	unsigned barpos;		/* starting position in menu bar */
	unsigned width;			/* width of menu */
	short	micolor;		/* menu item color */
	short	menucolor;		/* menu background color */
	unsigned menuid;		/* unique menu id code */
	struct hibox box;		/* hilight area */
	long 	size;			/* menu background size in bytes */
	short int ix1, iy1, ix2, iy2;	/* menu area in logical coords */
	char huge *background;		/* background image, NULL if none */
	struct menu *next;		/* next menu */
	struct menuitem *top;		/* entries in this menu */
	};
	
/* individual item under a menu */
struct menuitem
	{
	char	*entry;			/* menu item entry */
	unsigned hf;			/* hilight this item or not */
	unsigned itemid;		/* unique item id code */
	struct hibox box;		/* hilight area */
	struct menuitem *next;		/* next item in the list */
	};
	

/* raw data read from input file */
struct rawcell
	{
	double x;
	struct rawcell *next;
	};
	
/* bin cell structure */
struct bin
	{
	double	xval;		/* start of bin */
	double	x;		/* x point for fit */
	double	y;		/* size of bin */
	struct bin *next;	/* next bin */
	};
	
/* top of bin list */
struct topbin
	{
	int	dclass;				/* DISCRETE or CONTINUOUS */
	double	xmin, xmax;			/* x-axis extremes */
	double	ymin, ymax;			/* y-axis extremes */
	double	binsize;			/* size of cont. class bin */
	int	dbinsize;			/* size of disc. class bin */
	double	numpts;				/* total number of data pts. */
	double	mean, var;			/* mean and variance of data */
	struct bin *start;			/* start of the bin list */
	};


/* dialog box structures */
/* text line */
struct diatext
	{
	char *text;			/* text string */
	short int col, row;		/* starting position in char coords */
	struct diatext *next;		/* next string */
	};
	
/* edit box */
struct editbox
	{
	int	id;			/* id of this ebox */
	char 	*tbox;			/* edit box text */
	short int lx1, ly1;		/* logical coords of ulc of ebox */
	short int size;			/* char size of ebox */
	struct grafwindow *ebw;		/* window associated with this ebox */
	struct editbox *next;		/* point to next ebox */
	};

/* a dialog */
struct dialog
	{
	int	id;			/* dialog id */
	short int dlx1, dly1;		/* logical coords. of the upper left
					   corner of the dialog box. */
	short int dcols, drows;		/* number of text rows and columns */
	struct diatext *dtext;		/* text items */
	struct editbox *ebox;		/* edit boxes */
	struct dialog *next;		/* next dialog */
	};
	

/* memory monitor data structure */
struct mwdat
	{
	struct grafwindow *main;
	struct grafwindow *data;
	};
